package nl.tue.id.pp.boids;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

import java.util.ArrayList;
import processing.core.*;

/**
 * 
 * @author ppeters
 * 
 *         Example program using processing library to display many shapes that
 *         rotate themselves and also are rotated depending on the mouse pointer
 */
public class ManyBoidsExample extends PApplet {

	/*
	 *  final properties
	 */
	private static final long serialVersionUID = 1L;	// version
	private final boolean squareboid = false;			// display square or triangle

	/*
	 *  private properties
	 */
	private int xsize = 20; // number of boids horizontally
	private int ysize = 20; // number of boids vertically
	private int step = 20; // distance between boids
	private ArrayList<Boid> boids; // An arraylist to contain all the boids
	private boolean rotate = false; // Initially boids do not rotate

	/** setup drawing canvas (non-Javadoc)
	 * 
	 * @see processing.core.PApplet#setup()
	 */
	public void setup() {
        // create field that accomodates all boids
        size((xsize + 1) * step, (ysize + 1) * step);
        // refreshes per second...
        frameRate(30);
        // anti aliasing
        smooth();

        // create the boid container
        boids = new ArrayList<Boid>();
        // fill the container with boids evenly spaced
        for (int i = 1; i <= xsize; i++) {
            for (int j = 1; j <= ysize; j++) {
            	if (squareboid) {
            		boids.add(new SquareBoid(this, i * step, j * step, 0));
            	} else {
            		boids.add(new TriangleBoid(this, i * step, j * step, 0));
            	}
            }
        }
    }

	/** When the mouse is moved, focus all boids (non-Javadoc)
	 * 
	 * @see processing.core.PApplet#mouseMoved()
	 */
	public void mouseMoved() {
		for (int i = 0; i < boids.size(); i++) {
			boids.get(i).pointAt(mouseX, mouseY);
		}
	}

	/** When the mouse is pressed all boids start rotating (non-Javadoc)
	 * 
	 * @see processing.core.PApplet#mousePressed()
	 */
	public void mousePressed() {
		rotate = !rotate;
	}

	/*
	 * rotate all boids
	 */
	private void rotateBoids() {
		// for all boids... rotate em with some randomization...
		for (int i = 0; i < boids.size(); i++) {
			Boid b = boids.get(i);
			b.setAngle(b.getAngle() + random(0.2f));
		}
	}

	/*
	 * draw all boids
	 */
	private void drawBoids() {
		for (int i = 0; i < boids.size(); i++) {
			Boid b = boids.get(i);
			b.render();
		}
	}

	/** render all boids on screen (non-Javadoc)
	 * 
	 * @see processing.core.PApplet#draw()
	 */
	public void draw() {
		// set backgroundcolor
		background(102);
		// perform changes here before drawing...
		if (rotate) {
			rotateBoids();
		}
		// draw all boids
		drawBoids();
	}

	/** main entry point
	 * @param args
	 *            the command line arguments
	 */
	public static void main(String args[]) {
		PApplet.main(new String[] { "nl.tue.id.pp.boids.ManyBoidsExample" });
	}

}
